//-----------------------------------------------------------------------------
// AScript time module
//-----------------------------------------------------------------------------
#include "Module_time.h"
#include "Object_DateTime.h"

AScript_BeginModule(time)

//-----------------------------------------------------------------------------
// AScript module functions: time
//-----------------------------------------------------------------------------
// time.sleep(secs)
AScript_DeclareFunction(sleep)
{
	SetMode(RSLTMODE_Normal, MAP_Off, FLAT_Off);
	DeclareArg(env, "secs", VTYPE_Number);
}

#if defined(HAVE_WINDOWS_H)
AScript_ImplementFunction(sleep)
{
	::Sleep(static_cast<DWORD>(context.GetNumber(0) * 1000));
	return Value::Null;
}
#else
AScript_ImplementFunction(sleep)
{
	return Value::Null;
}
#endif

// time.gmtime(t?:number)
AScript_DeclareFunction(gmtime)
{
	SetMode(RSLTMODE_Normal, MAP_Off, FLAT_Off);
	//DeclareArg(env, "t", VTYPE_Number, false, OCCUR_ZeroOrOnce);
}

#if defined(HAVE_WINDOWS_H)
AScript_ImplementFunction(gmtime)
{
	if (context.IsNumber(0)) {
		return Value::Null;
	} else {
		SYSTEMTIME st;
		::GetSystemTime(&st);
		Object_DateTime *pObj = new Object_DateTime(env.GetClass_DateTime(), DateTime(st));
		return Value(pObj, VTYPE_DateTime);
	}
}
#else
AScript_ImplementFunction(gmtime)
{
	return Value::Null;
}
#endif

// time.localtime(t?:number)
AScript_DeclareFunction(localtime)
{
	SetMode(RSLTMODE_Normal, MAP_Off, FLAT_Off);
	//DeclareArg(env, "t", VTYPE_Number, false, OCCUR_ZeroOrOnce);
}

#if defined(HAVE_WINDOWS_H)
AScript_ImplementFunction(localtime)
{
	if (context.IsNumber(0)) {
		return Value::Null;
	} else {
		SYSTEMTIME st;
		::GetLocalTime(&st);
		Object_DateTime *pObj = new Object_DateTime(env.GetClass_DateTime(), DateTime(st));
		return Value(pObj, VTYPE_DateTime);
	}
}
#else
AScript_ImplementFunction(localtime)
{
	return Value::Null;
}
#endif

// time.clock()
AScript_DeclareFunction(clock)
{
	SetMode(RSLTMODE_Normal, MAP_Off, FLAT_Off);
}

#if defined(HAVE_WINDOWS_H)
AScript_ImplementFunction(clock)
{
	LARGE_INTEGER freq, counter;
	if (::QueryPerformanceFrequency(&freq) && ::QueryPerformanceCounter(&counter)) {
		return Value(static_cast<Number>(counter.QuadPart) / freq.QuadPart);
	} else {
		return Value(static_cast<Number>(::GetTickCount()) / 1000);
	}
}
#else
AScript_ImplementFunction(clock)
{
	return Value::Null;
}
#endif

AScript_ModuleEntry()
{
	AScript_AssignFunction(sleep);
	AScript_AssignFunction(gmtime);
	AScript_AssignFunction(localtime);
	AScript_AssignFunction(clock);
}

AScript_ModuleTerminate()
{
}

AScript_EndModule(time)

AScript_DLLModuleEntry(time)
